precision mediump float;

uniform sampler2D tex;
uniform vec2 uOutputPixelSize;
varying vec2 varyTexCoord;

uniform float uBorderMirror;
uniform float uBorderRepeat;

#define BORDER_EPS 0.001

// Brightness function
float Brightness(vec3 c)
{
	return max(max(c.r, c.g), c.b);
}

// should have a "partial repeat" instead?
vec2 repeatAndMirrorTex(vec2 t)
{
    t = t.x > (1.0-BORDER_EPS) ?
		vec2(mix(t.x, t.x - 1.0, uBorderRepeat), mix(t.y, 1.0 - t.y, uBorderMirror))
		: (t.x < BORDER_EPS ? vec2(mix(t.x, t.x + 1.0, uBorderRepeat), mix(t.y, 1.0 - t.y, uBorderMirror)) : t);
    return t;
}

vec4 zeroBorderSample(vec4 sample, vec2 t)
{
    float borderZero = 1.0 - uBorderRepeat;
    sample = (t.x > (1.0-BORDER_EPS) || t.x < BORDER_EPS) ? (mix(sample, vec4(0.0), borderZero) + sample) * 0.5 : sample;
    sample = (t.y > (1.0-BORDER_EPS) || t.y < BORDER_EPS) ? (mix(sample, vec4(0.0), borderZero) + sample) * 0.5 : sample;
    return sample;
}

vec4 sampleBlurTex(vec2 coords)
{
	coords = repeatAndMirrorTex(coords);
	coords = repeatAndMirrorTex(coords.yx).yx;
	vec4 sample = texture2D(tex, coords);
	//if(uPixelStep.y > 0.0)
	//{
	//if( coords.x < 0.55  && coords.x > 0.45 && coords.y < 0.55  && coords.y > 0.45) sample = vec3(1.0);
	//else sample = vec3(0.0);
	//}
	return zeroBorderSample(sample, coords);
}

void main()
{
	vec4 offsets = uOutputPixelSize.xyxy * 0.5 * vec4(-1.0, -1.0, 1.0, 1.0);
	vec4 s1 = sampleBlurTex(varyTexCoord + offsets.xy);
	vec4 s2 = sampleBlurTex(varyTexCoord + offsets.zy);
	vec4 s3 = sampleBlurTex(varyTexCoord + offsets.xw);
	vec4 s4 = sampleBlurTex(varyTexCoord + offsets.zw);
	
	/* // Not sure why I was not using it
	// Karis's luma weighted average
	float s1w = 1.0 / (Brightness(s1) + 1.0);
	float s2w = 1.0 / (Brightness(s2) + 1.0);
	float s3w = 1.0 / (Brightness(s3) + 1.0);
	float s4w = 1.0 / (Brightness(s4) + 1.0);
	float one_div_wsum = 1.0 / (s1w + s2w + s3w + s4w);

	//vec3 final = (s1 * s1w + s2 * s2w + s3 * s3w + s4 * s4w) * one_div_wsum;
	*/
	vec4 final = (s1 + s2 + s3 + s4) * 0.25;
	
	gl_FragColor = final;
}
